function [] = runDiagonal(RestartLevel)

syms x xh y yh a ah b bh;
X = [x xh y yh a ah b bh];

syms x0 x0h y0 y0h a0 a0h b0 b0h g0 g0h;
XD = [x0 x0h y0 y0h a0 a0h b0 b0h];
GD = [g0 g0h];

syms g1 gh1 g2 gh2 g3 gh3 g4 gh4 g5 gh5 g6 gh6 g7 gh7 g8 gh8;
G = [g1 gh1 g2 gh2 g3 gh3 g4 gh4 g5 gh5 g6 gh6 g7 gh7 g8 gh8];

if nargin == 0 || RestartLevel < 2
  % setup slices 
  if exist('slices_A') > 0
    [~,A] = ReadInStart(8,'slices_A');
  else
    A = rand(8,8)*2 - 1 + 1i*(rand(8,8)*2 - 1);
    CreateStart(A,8,8,'slices_A');
  end;

  % setup diagonal slices 
  AD = rand(8,8)*2 - 1 + 1i*(rand(8,8)*2 - 1);
  CreateStart(AD,8,8,'slices_AD');

  % solve diagonal using AD
  solveDiagonal(AD,G,X);
  eval(['!mv finite_solutions start_diagonal']);

  % solve first system for A
  solveFirst(A,G,X);

  % solve k = 2,..,8 systems (degenerate test when k = 9)
  for k = 2:9
    solveSystem(k,A,AD,G,GD,X,XD);
  end;
else
  [~,A] = ReadInStart(8,'slices_A');
  [~,AD] = ReadInStart(8,'slices_AD');

  disp(' ');
  disp(['Solving system ',num2str(RestartLevel)]);
  eval(['!mpirun bertini input_',num2str(RestartLevel),' start_',num2str(RestartLevel),' > /dev/null']);

  for k = RestartLevel+1:9 % degenerate test when k = 9
    solveSystem(k,A,AD,G,GD,X,XD);
  end;
end;

return;

function [] = solveSystem(k,A,AD,G,GD,X,XD)

% move finite_solutions
eval(['!mv finite_solutions solutions_',num2str(k-1)]);

if k > 2
  % setup proper coordinates
  [n,P] = ReadInStart(16+2*(k-1),['solutions_',num2str(k-1)]);
  P = P(:,[11:end,9,10]);
  CreateStart(P,n,8+2*(k-1),['solutions_',num2str(k-1)]);
end;

% perform degenerate test
degenerateTest(k-1,G);

if k > 8 % only perform degenerate test
  return;
end;

[n,P] = ReadInStart(8+2*(k-1),['solutions_',num2str(k-1)]);
disp(['Solutions from system ',num2str(k-1),': ',num2str(n)]);
disp(' ');

% read in points from diagonal
[nD,D] = ReadInStart(10,'start_diagonal');

% setup new start points
Q = zeros(n*nD,16+2*k);
c = 1;
for j = 1:n
  for l = 1:nD
    Q(c,:) = [D(l,:) P(j,:)];
    c = c + 1;
  end;
end;

% print to file
CreateStart(Q,n*nD,16+2*k,['start_',num2str(k)]);

% create system
F = fopen('input_bottom','w');
fprintf(F,'pathvariable t;\nparameter s;\ns = t;\n');
% setup variables and main functions
for l = 1:k-1
  fprintf(F,'variable_group %s,%s;\n',char(G(2*(l-1)+1)),char(G(2*(l-1)+2)));
  fprintf(F,'function G%d1,G%d2,G%d3;\n',l,l,l);
  printPoly3(l,F);
end;
for l = k:8
  fprintf(F,'function l%d;\n',l);
end;
% setup constants
printConstants(k-1,A,F);
printDiagonalConstants(AD,F);
% setup fixed linears
printLinear(k,A,F,X);

% print diagonal functions
printPoly0(k,F);

% setup moving polynomials
fprintf(F,'l%d = (',k);
for l = 1:8
  fprintf(F,'r%d%d*%s + ',k,l,char(X(l)));
end;
fprintf(F,' - 1)*s + (1-s)*(%s - %s);\n',char(X(1)),char(XD(1)));

for j = 2:8
  fprintf(F,'L%d = (',j);
  for l = 1:8
    fprintf(F,'R%d%d*%s + ',j,l,char(XD(l)));
  end;
  fprintf(F,' - 1)*s + (1-s)*(%s - %s);\n',char(X(j)),char(XD(j)));
end;

fprintf(F,'END;\n');
fclose(F);

disp(' ');
disp(['Solving system ',num2str(k),': ',num2str(n*nD),' paths']);
eval(['!rm -rf input_',num2str(k),'; cat input_top input_bottom > input_',num2str(k),'; mpirun bertini input_',num2str(k),' start_',num2str(k),' > /dev/null']);

return;

function [] = solveFirst(A,G,X)

% open file
F = fopen('input_1_bottom','w');

% setup variables
fprintf(F, 'variable_group %s,%s;\n',char(G(1)),char(G(2)));

% setup constants
printConstants(1,A,F);

% setup polynomials
fprintf(F, 'function G11,G12,G13,l2,l3,l4,l5,l6,l7,l8;\n');
printPoly3(1,F);
printLinear(1,A,F,X);
fprintf(F,'END;\n');
fclose(F);

disp(['Solving system 1']);
eval(['!rm -rf input_1; cat input_1_top input_1_bottom > input_1; mpirun bertini input_1 > /dev/null']);

return;

function [] = solveDiagonal(AD,G,X)

% open file
F = fopen('input_diagonal_bottom','w');
fprintf(F, 'variable_group %s,%s;\n',char(G(1)),char(G(2)));

% setup constants
printConstants(1,AD,F);

% setup polynomials
fprintf(F, 'function G11,G12,G13,l2,l3,l4,l5,l6,l7,l8;\n');
printPoly3(1,F);
printLinear(1,AD,F,X);
fprintf(F,'END;\n');
fclose(F);

disp(['Solving diagonal']);
eval(['!rm -rf input_diagonal; cat input_diagonal_top input_diagonal_bottom > input_diagonal; mpirun bertini input_diagonal > /dev/null']);

return;

function [] = printConstants(k,A,F)

c = 'constant ';
for j = k+1:8
  for l = 1:8
    c = [c,'r',num2str(j),num2str(l)];
    if j == 8 && l == 8
      c = [c,';'];
    else
      c = [c,','];
    end;
  end;
end;
fprintf(F,'%s\n',c);

for j = k+1:8
  for l = 1:8
    fprintf(F,'r%d%d = %s + I*%s;\n',j,l,char(vpa(real(A(j,l)))),char(vpa(imag(A(j,l)))));
  end;
end;

return;

function [] = printDiagonalConstants(AD,F)

c = 'constant ';
for j = 2:8
  for l = 1:8
    c = [c,'R',num2str(j),num2str(l)];
    if j == 8 && l == 8
      c = [c,';'];
    else
      c = [c,','];
    end;
  end;
end;
fprintf(F,'%s\n',c);

for j = 2:8
  for l = 1:8
    fprintf(F,'R%d%d = %s + I*%s;\n',j,l,char(vpa(real(AD(j,l)))),char(vpa(imag(AD(j,l)))));
  end;
end;

return;

function [] = printLinear(k,A,F,X)

for j = k+1:8
  c = ['l',num2str(j),' = '];
  for l = 1:8 
    c = [c,'r',num2str(j),num2str(l),'*',char(X(l))];
    if l < 8
      c = [c,' + '];
    else
      c = [c,' - 1;'];
    end;
  end;
  fprintf(F,'%s\n',c);
end;

return;

function [] = printPoly3(k,F)

fprintf(F,'G%d1 = (nh - dh%d*x)*g%d + (n - d%d*xh)*gh%d + (d%d*(ah - xh) + dh%d*(a - x) - d%d*dh%d);\n',k,k,k,k,k,k,k,k,k);
fprintf(F,'G%d2 = (mh - dh%d*y)*g%d + (m - d%d*yh)*gh%d + (d%d*(bh - yh) + dh%d*(b - y) - d%d*dh%d);\n',k,k,k,k,k,k,k,k,k);
fprintf(F,'G%d3 = g%d*gh%d + g%d + gh%d;\n',k,k,k,k,k);

return;

function [] = printPoly0(k,F)

fprintf(F,'G01 = (n0h - (Dh0*s + (1-s)*dh%d)*x0)*g0 + (n0 - (D0*s + (1-s)*d%d)*x0h)*g0h + ((D0*s + (1-s)*d%d)*(a0h - x0h) + (Dh0*s + (1-s)*dh%d)*(a0 - x0) - (D0*s + (1-s)*d%d)*(Dh0*s + (1-s)*dh%d));\n',k,k,k,k,k,k);
fprintf(F,'G02 = (m0h - (Dh0*s + (1-s)*dh%d)*y0)*g0 + (m0 - (D0*s + (1-s)*d%d)*y0h)*g0h + ((D0*s + (1-s)*d%d)*(b0h - y0h) + (Dh0*s + (1-s)*dh%d)*(b0 - y0) - (D0*s + (1-s)*d%d)*(Dh0*s + (1-s)*dh%d));\n',k,k,k,k,k,k);
fprintf(F,'G03 = g0*g0h + g0 + g0h;\n');

return;


function [] = degenerateTest(k,G)

tol = 1e-9;

% setup variable list
F = fopen('input_degenerate_middle','w');
for j = 1:k
  fprintf(F,'variable_group %s,%s;\n',char(G(2*(j-1)+1)),char(G(2*(j-1)+2)));
end;
fclose(F);

disp(['Running degenerate test ',num2str(k)]);
eval(['!rm -rf input_degenerate_',num2str(k),'; cat input_degenerate_top input_degenerate_middle input_degenerate_bottom > input_degenerate_',num2str(k),'; bertini input_degenerate_',num2str(k),' solutions_',num2str(k),' > /dev/null']);

[n,F] = ReadInStart(12,'function');
Good = n;

if min(min(abs(F))) < tol
  % we need to read in the file and remove the degenerate ones
  [n,P] = ReadInStart(8+2*k,['solutions_',num2str(k)]);
  G = ones(1,n);
  for j = 1:n
    if min(abs(F(j,:))) < tol
      G(j) = 0;
    end;
  end;
  PG = P(G > 0,:);
  Good = size(PG,1);
  CreateStart(PG,Good,8+2*k,['solutions_',num2str(k)]);
end;

disp(['Degenerate test ',num2str(k),': ',num2str(Good),' of ',num2str(n),' are nondegenerate']);

return;



